/*
 * artoolkit.c
 */

#include "artoolkit.h"

ScmClass *ARPtrClass;

static void Scm_finalize_ArMarkerInfo(ScmObj obj, void *data)
{
    ScmArMarkerInfo *z = SCM_AR_MARKER_INFO(obj);
    if (z->data) {
        z->data = NULL;
    }
}

ScmObj Scm_MakeArMarkerInfo(ARMarkerInfo *data)
{
    ScmArMarkerInfo *z = SCM_NEW(ScmArMarkerInfo);
    SCM_SET_CLASS(z, SCM_CLASS_AR_MARKER_INFO);
    Scm_RegisterFinalizer(SCM_OBJ(z), Scm_finalize_ArMarkerInfo, NULL);
    z->data = data;

    SCM_RETURN(SCM_OBJ(z));
}

static void Scm_finalize_ArMarkerInfo2(ScmObj obj, void *data)
{
    ScmArMarkerInfo2 *z = SCM_AR_MARKER_INFO2(obj);
    if (z->data) {
        z->data = NULL;
    }
}

ScmObj Scm_MakeArMarkerInfo2(ARMarkerInfo2 *data)
{
    ScmArMarkerInfo2 *z = SCM_NEW(ScmArMarkerInfo2);
    SCM_SET_CLASS(z, SCM_CLASS_AR_MARKER_INFO2);
    Scm_RegisterFinalizer(SCM_OBJ(z), Scm_finalize_ArMarkerInfo2, NULL);
    z->data = data;

    SCM_RETURN(SCM_OBJ(z));
}

static void Scm_finalize_ArMat(ScmObj obj, void *data)
{
    ScmArMat *z = SCM_AR_MAT(obj);
    if (z->data) {
        arMatrixFree(z->data);
        z->data = NULL;
    }
}

ScmObj Scm_MakeArMat(ARMat *data)
{
    ScmArMat *z = SCM_NEW(ScmArMat);
    SCM_SET_CLASS(z, SCM_CLASS_AR_MAT);
    Scm_RegisterFinalizer(SCM_OBJ(z), Scm_finalize_ArMat, NULL);
    z->data = data;

    SCM_RETURN(SCM_OBJ(z));
}

static void Scm_finalize_ArMultiEachMarkerInfoT(ScmObj obj, void *data)
{
    ScmArMultiEachMarkerInfoT *z = SCM_AR_MULTI_EACH_MARKER_INFO_T(obj);
    if (z->data) {
        free(z->data);
        z->data = NULL;
    }
}

ScmObj Scm_MakeArMultiEachMarkerInfoT(ARMultiEachMarkerInfoT *data)
{
    ScmArMultiEachMarkerInfoT *z = SCM_NEW(ScmArMultiEachMarkerInfoT);
    SCM_SET_CLASS(z, SCM_CLASS_AR_MULTI_EACH_MARKER_INFO_T);
    Scm_RegisterFinalizer(SCM_OBJ(z), Scm_finalize_ArMultiEachMarkerInfoT, NULL);
    z->data = data;

    SCM_RETURN(SCM_OBJ(z));
}

static void Scm_finalize_ArMultiMarkerInfoT(ScmObj obj, void *data)
{
    ScmArMultiMarkerInfoT *z = SCM_AR_MULTI_MARKER_INFO_T(obj);
    if (z->data) {
        arMultiFreeConfig(z->data);
        z->data = NULL;
    }
}

ScmObj Scm_MakeArMultiMarkerInfoT(ARMultiMarkerInfoT *data)
{
    ScmArMultiMarkerInfoT *z = SCM_NEW(ScmArMultiMarkerInfoT);
    SCM_SET_CLASS(z, SCM_CLASS_AR_MULTI_MARKER_INFO_T);
    Scm_RegisterFinalizer(SCM_OBJ(z), Scm_finalize_ArMultiMarkerInfoT, NULL);
    z->data = data;

    SCM_RETURN(SCM_OBJ(z));
}

static void Scm_finalize_ArParam(ScmObj obj, void *data)
{
    ScmArParam *z = SCM_AR_PARAM(obj);
    if (z->data) {
        /* ARParam is allocated with SCM_NEW (in make-ar-param)
           , so don't free manually */
        z->data = NULL;
    }
}

ScmObj Scm_MakeArParam(ARParam *data)
{
    ScmArParam *z = SCM_NEW(ScmArParam);
    SCM_SET_CLASS(z, SCM_CLASS_AR_PARAM);
    Scm_RegisterFinalizer(SCM_OBJ(z), Scm_finalize_ArParam, NULL);
    z->data = data;

    SCM_RETURN(SCM_OBJ(z));
}

static void Scm_finalize_ArPrevInfo(ScmObj obj, void *data)
{
    ScmArPrevInfo *z = SCM_AR_PREV_INFO(obj);
    if (z->data) {
        free(z->data);
        z->data = NULL;
    }
}

ScmObj Scm_MakeArPrevInfo(arPrevInfo *data)
{
    ScmArPrevInfo *z = SCM_NEW(ScmArPrevInfo);
    SCM_SET_CLASS(z, SCM_CLASS_AR_PREV_INFO);
    Scm_RegisterFinalizer(SCM_OBJ(z), Scm_finalize_ArPrevInfo, NULL);
    z->data = data;

    SCM_RETURN(SCM_OBJ(z));
}

static void Scm_finalize_ArVec(ScmObj obj, void *data)
{
    ScmArVec *z = SCM_AR_VEC(obj);
    if (z->data) {
        arVecFree(z->data);
        free(z->data);
        z->data = NULL;
    }
}

ScmObj Scm_MakeArVec(ARVec *data)
{
    ScmArVec *z = SCM_NEW(ScmArVec);
    SCM_SET_CLASS(z, SCM_CLASS_AR_VEC);
    Scm_RegisterFinalizer(SCM_OBJ(z), Scm_finalize_ArVec, NULL);
    z->data = data;

    SCM_RETURN(SCM_OBJ(z));
}

static void Scm_finalize_ArglContextSettingsRef(ScmObj obj, void *data)
{
    ScmArglContextSettingsRef *z = SCM_ARGL_CONTEXT_SETTINGS_REF(obj);
    if (z->data) {
        free(z->data);
        z->data = NULL;
    }
}

ScmObj Scm_MakeArglContextSettingsRef(ARGL_CONTEXT_SETTINGS_REF *data)
{
    ScmArglContextSettingsRef *z = SCM_NEW(ScmArglContextSettingsRef);
    SCM_SET_CLASS(z, SCM_CLASS_ARGL_CONTEXT_SETTINGS_REF);
    Scm_RegisterFinalizer(SCM_OBJ(z), Scm_finalize_ArglContextSettingsRef, NULL);
    z->data = data;

    SCM_RETURN(SCM_OBJ(z));
}


static ScmObj mouseProc;
static ScmObj keyProc;
static ScmObj mainProc;

static void mouseFunc(int button, int state, int x, int y)
{
    ScmEvalPacket epak;

    if (SCM_FALSEP(mouseProc)) {
        return;
    }

    Scm_Apply(mouseProc, SCM_LIST4(SCM_MAKE_INT(button)
                                   , SCM_MAKE_INT(state)
                                   , SCM_MAKE_INT(x)
                                   , SCM_MAKE_INT(y))
              , &epak);
    if (!SCM_FALSEP(epak.exception)) {
        Scm_Raise(epak.exception);
    }
}

static void keyFunc(unsigned char key, int x, int y)
{
    ScmEvalPacket epak;

    if (SCM_FALSEP(keyProc)) {
        return;
    }

    Scm_Apply(keyProc, SCM_LIST3(SCM_MAKE_INT(key) , SCM_MAKE_INT(x) , SCM_MAKE_INT(y))
              , &epak);
    if (!SCM_FALSEP(epak.exception)) {
        Scm_Raise(epak.exception);
    }
}

static void mainFunc()
{
    ScmEvalPacket epak;

    if (SCM_FALSEP(mainProc)) {
        return;
    }

    Scm_Apply(mainProc, SCM_NIL, &epak);
    if (!SCM_FALSEP(epak.exception)) {
        Scm_Raise(epak.exception);
    }
}

ScmObj Scm_argInqSetting()
{
    int hmdMode;
    int gMiniXnum2;
    int gMiniYnum2;
    void (*mFunc)(int button, int state, int x, int y);
    void (*kFunc)(unsigned char key, int x, int y);
    void (*mnFunc)(void);

    argInqSetting(&hmdMode, &gMiniXnum2, &gMiniYnum2, &mFunc, &kFunc, &mnFunc);

    SCM_RETURN(Scm_Values(Scm_Cons(SCM_MAKE_INT(hmdMode)
                                   , SCM_LIST5(SCM_MAKE_INT(gMiniXnum2)
                                               , SCM_MAKE_INT(gMiniYnum2)
                                               , mouseProc
                                               , keyProc
                                               , mainProc))));
}

ScmObj Scm_argMainLoop(ScmObj mouse_proc , ScmObj key_proc , ScmObj main_proc)
{
    if (!(SCM_PROCEDUREP(mouse_proc) || SCM_FALSEP(mouse_proc))) {
        Scm_Error("procedure or #f required, but got %S", mouse_proc);
    }
    if (!(SCM_PROCEDUREP(key_proc) || SCM_FALSEP(key_proc))) {
        Scm_Error("procedure or #f required, but got %S", key_proc);
    }
    if (!(SCM_PROCEDUREP(main_proc) || SCM_FALSEP(main_proc))) {
        Scm_Error("procedure or #f required, but got %S", main_proc);
    }
    mouseProc = mouse_proc;
    keyProc = key_proc;
    mainProc = main_proc;

    argMainLoop(mouseFunc, keyFunc, mainFunc);

    SCM_RETURN(SCM_UNDEFINED);
}

static ScmObj postProc;

static void postFunc(ARParam *lpara, ARParam *rpara)
{
    ScmEvalPacket epak;
    ScmObj lparaObj = SCM_FALSE;
    ScmObj rparaObj = SCM_FALSE;

    if (SCM_FALSEP(postProc)) {
        return;
    }
    
    if (lpara != NULL) {
        lparaObj = SCM_MAKE_AR_PARAM(lpara);
    }
    if (rpara != NULL) {
        rparaObj = SCM_MAKE_AR_PARAM(rpara);
    }
    Scm_Apply(postProc, SCM_LIST2(lparaObj , rparaObj) , &epak);
    if (!SCM_FALSEP(epak.exception)) {
        Scm_Raise(epak.exception);
    }
}

ScmObj Scm_argUtilCalibHMD(int targetId, int thresh2, ScmObj post_proc)
{
    if (!(SCM_PROCEDUREP(post_proc) || SCM_FALSEP(post_proc))) {
        Scm_Error("procedure or #f required, but got %S", post_proc);
    }
    postProc = post_proc;

    argUtilCalibHMD(targetId, thresh2, postFunc);

    SCM_RETURN(SCM_UNDEFINED);
}
    
/*
 * Module initialization function.
 */
extern void Scm_Init_artoolkitlib(ScmModule*);

void Scm_Init_artoolkit(void)
{
    ScmModule *mod;

    /* Register this DSO to Gauche */
    SCM_INIT_EXTENSION(artoolkit);

    /* Create the module if it doesn't exist yet. */
    mod = SCM_MODULE(SCM_FIND_MODULE("artoolkit", TRUE));

    /* Register stub-generated procedures */
    Scm_Init_artoolkitlib(mod);

    ARPtrClass = Scm_MakeForeignPointerClass(mod, "<ar-ptr>", NULL, NULL, SCM_FOREIGN_POINTER_KEEP_IDENTITY);
    mouseProc = SCM_FALSE;
    keyProc = SCM_FALSE;
    mainProc = SCM_FALSE;
}
