/*
 * artoolkit.h
 */

/* Prologue */
#ifndef GAUCHE_ARTOOLKIT_H
#define GAUCHE_ARTOOLKIT_H

#include <gauche.h>
#include <gauche/class.h>
#include <gauche/extend.h>
#include <gauche/uvector.h>

#include <AR/ar.h>
#include <AR/arMulti.h>
#include <AR/gsub.h>
#include <AR/gsub_lite.h>
#include <AR/gsubUtil.h>
#include <AR/matrix.h>
#include <AR/param.h>
#include <AR/video.h>

SCM_DECL_BEGIN

/* ARMarkerInfo */
typedef struct ScmArMarkerInfoRec {
    SCM_HEADER;
    ARMarkerInfo *data;
} ScmArMarkerInfo;

SCM_CLASS_DECL(Scm_ArMarkerInfoClass);
#define SCM_CLASS_AR_MARKER_INFO (&Scm_ArMarkerInfoClass)
#define SCM_AR_MARKER_INFO(obj) ((ScmArMarkerInfo*)(obj))
#define SCM_AR_MARKER_INFO_P(obj) SCM_XTYPEP(obj, SCM_CLASS_AR_MARKER_INFO)
#define SCM_AR_MARKER_INFO_DATA(obj) (SCM_AR_MARKER_INFO(obj)->data)
#define SCM_MAKE_AR_MARKER_INFO(data) (Scm_MakeArMarkerInfo(data))

extern ScmObj Scm_MakeArMarkerInfo(ARMarkerInfo *data);

/* ARMarkerInfo2 */
typedef struct ScmArMarkerInfo2Rec {
    SCM_HEADER;
    ARMarkerInfo2 *data;
} ScmArMarkerInfo2;

SCM_CLASS_DECL(Scm_ArMarkerInfo2Class);
#define SCM_CLASS_AR_MARKER_INFO2 (&Scm_ArMarkerInfo2Class)
#define SCM_AR_MARKER_INFO2(obj) ((ScmArMarkerInfo2*)(obj))
#define SCM_AR_MARKER_INFO2_P(obj) SCM_XTYPEP(obj, SCM_CLASS_AR_MARKER_INFO2)
#define SCM_AR_MARKER_INFO2_DATA(obj) (SCM_AR_MARKER_INFO2(obj)->data)
#define SCM_MAKE_AR_MARKER_INFO2(data) (Scm_MakeArMarkerInfo2(data))

extern ScmObj Scm_MakeArMarkerInfo2(ARMarkerInfo2 *data);

/* ARMat */
typedef struct ScmArMatRec {
    SCM_HEADER;
    ARMat *data;
} ScmArMat;

SCM_CLASS_DECL(Scm_ArMatClass);
#define SCM_CLASS_AR_MAT (&Scm_ArMatClass)
#define SCM_AR_MAT(obj) ((ScmArMat*)(obj))
#define SCM_AR_MAT_P(obj) SCM_XTYPEP(obj, SCM_CLASS_AR_MAT)
#define SCM_AR_MAT_DATA(obj) (SCM_AR_MAT(obj)->data)
#define SCM_MAKE_AR_MAT(data) (Scm_MakeArMat(data))

extern ScmObj Scm_MakeArMat(ARMat *data);

/* ARMultiEachMarkerInfoT */
typedef struct ScmArMultiEachMarkerInfoTRec {
    SCM_HEADER;
    ARMultiEachMarkerInfoT *data;
} ScmArMultiEachMarkerInfoT;

SCM_CLASS_DECL(Scm_ArMultiEachMarkerInfoTClass);
#define SCM_CLASS_AR_MULTI_EACH_MARKER_INFO_T (&Scm_ArMultiEachMarkerInfoTClass)
#define SCM_AR_MULTI_EACH_MARKER_INFO_T(obj) ((ScmArMultiEachMarkerInfoT*)(obj))
#define SCM_AR_MULTI_EACH_MARKER_INFO_T_P(obj) SCM_XTYPEP(obj, SCM_CLASS_AR_MULTI_EACH_MARKER_INFO_T)
#define SCM_AR_MULTI_EACH_MARKER_INFO_T_DATA(obj) (SCM_AR_MULTI_EACH_MARKER_INFO_T(obj)->data)
#define SCM_MAKE_AR_MULTI_EACH_MARKER_INFO_T(data) (Scm_MakeArMultiEachMarkerInfoT(data))

extern ScmObj Scm_MakeArMultiEachMarkerInfoT(ARMultiEachMarkerInfoT *data);

/* ARMultiMarkerInfoT */
typedef struct ScmArMultiMarkerInfoTRec {
    SCM_HEADER;
    ARMultiMarkerInfoT *data;
} ScmArMultiMarkerInfoT;

SCM_CLASS_DECL(Scm_ArMultiMarkerInfoTClass);
#define SCM_CLASS_AR_MULTI_MARKER_INFO_T (&Scm_ArMultiMarkerInfoTClass)
#define SCM_AR_MULTI_MARKER_INFO_T(obj) ((ScmArMultiMarkerInfoT*)(obj))
#define SCM_AR_MULTI_MARKER_INFO_T_P(obj) SCM_XTYPEP(obj, SCM_CLASS_AR_MULTI_MARKER_INFO_T)
#define SCM_AR_MULTI_MARKER_INFO_T_DATA(obj) (SCM_AR_MULTI_MARKER_INFO_T(obj)->data)
#define SCM_MAKE_AR_MULTI_MARKER_INFO_T(data) (Scm_MakeArMultiMarkerInfoT(data))

extern ScmObj Scm_MakeArMultiMarkerInfoT(ARMultiMarkerInfoT *data);

/* ARParam */
typedef struct ScmArParamRec {
    SCM_HEADER;
    ARParam *data;
} ScmArParam;

SCM_CLASS_DECL(Scm_ArParamClass);
#define SCM_CLASS_AR_PARAM (&Scm_ArParamClass)
#define SCM_AR_PARAM(obj) ((ScmArParam*)(obj))
#define SCM_AR_PARAM_P(obj) SCM_XTYPEP(obj, SCM_CLASS_AR_PARAM)
#define SCM_AR_PARAM_DATA(obj) (SCM_AR_PARAM(obj)->data)
#define SCM_MAKE_AR_PARAM(data) (Scm_MakeArParam(data))

extern ScmObj Scm_MakeArParam(ARParam *data);

/* arPrevInfo */
typedef struct ScmArPrevInfoRec {
    SCM_HEADER;
    arPrevInfo *data;
} ScmArPrevInfo;

SCM_CLASS_DECL(Scm_ArPrevInfoClass);
#define SCM_CLASS_AR_PREV_INFO (&Scm_ArPrevInfoClass)
#define SCM_AR_PREV_INFO(obj) ((ScmArPrevInfo*)(obj))
#define SCM_AR_PREV_INFO_P(obj) SCM_XTYPEP(obj, SCM_CLASS_AR_PREV_INFO)
#define SCM_AR_PREV_INFO_DATA(obj) (SCM_AR_PREV_INFO(obj)->data)
#define SCM_MAKE_AR_PREV_INFO(data) (Scm_MakeArPrevInfo(data))

extern ScmObj Scm_MakeArPrevInfo(arPrevInfo *data);

/* ARVec */
typedef struct ScmArVecRec {
    SCM_HEADER;
    ARVec *data;
} ScmArVec;

SCM_CLASS_DECL(Scm_ArVecClass);
#define SCM_CLASS_AR_VEC (&Scm_ArVecClass)
#define SCM_AR_VEC(obj) ((ScmArVec*)(obj))
#define SCM_AR_VEC_P(obj) SCM_XTYPEP(obj, SCM_CLASS_AR_VEC)
#define SCM_AR_VEC_DATA(obj) (SCM_AR_VEC(obj)->data)
#define SCM_MAKE_AR_VEC(data) (Scm_MakeArVec(data))

extern ScmObj Scm_MakeArVec(ARVec *data);

/* ARGL_CONTEXT_SETTINGS_REF */
typedef struct ScmArglContextSettingsRefRec {
    SCM_HEADER;
    ARGL_CONTEXT_SETTINGS_REF *data;
} ScmArglContextSettingsRef;

SCM_CLASS_DECL(Scm_ArglContextSettingsRefClass);
#define SCM_CLASS_ARGL_CONTEXT_SETTINGS_REF (&Scm_ArglContextSettingsRefClass)
#define SCM_ARGL_CONTEXT_SETTINGS_REF(obj) ((ScmArglContextSettingsRef*)(obj))
#define SCM_ARGL_CONTEXT_SETTINGS_REF_P(obj) SCM_XTYPEP(obj, SCM_CLASS_ARGL_CONTEXT_SETTINGS_REF)
#define SCM_ARGL_CONTEXT_SETTINGS_REF_DATA(obj) (SCM_ARGL_CONTEXT_SETTINGS_REF(obj)->data)
#define SCM_MAKE_ARGL_CONTEXT_SETTINGS_REF(data) (Scm_MakeArglContextSettingsRef(data))

extern ScmObj Scm_MakeArglContextSettingsRef(ARGL_CONTEXT_SETTINGS_REF *data);


extern ScmClass *ARPtrClass;

extern ScmObj Scm_argInqSetting();
extern ScmObj Scm_argMainLoop(ScmObj mouse_proc , ScmObj key_proc , ScmObj main_proc);
extern ScmObj Scm_argUtilCalibHMD(int targetId, int thresh2, ScmObj post_proc);

#define S32VECTOR_MEMCPY(dest, src, len)     \
    do { \
        if (SCM_S32VECTOR_SIZE(dest) < len) { \
            Scm_Error("s32vector size must be %S, but got %S" \
                      , len, SCM_S32VECTOR_SIZE(dest)); \
        } else { \
            memcpy(SCM_S32VECTOR_ELEMENTS(dest), src, len*sizeof(int)); \
        } \
    } while (0)
#define F64VECTOR_MEMCPY(dest, src, len)     \
    do { \
        if (SCM_F64VECTOR_SIZE(dest) < len) { \
            Scm_Error("f64vector size must be %S, but got %S" \
                      , len, SCM_F64VECTOR_SIZE(dest)); \
        } else { \
            memcpy(SCM_F64VECTOR_ELEMENTS(dest), src, len*sizeof(double)); \
        } \
    } while (0)

/* Epilogue */
SCM_DECL_END

#endif  /* GAUCHE_ARTOOLKIT_H */
